%% Microstructure simulation
% by Vaclav Nezerka 1/6/2013

% addpath(fullfile(pwd,'output'))
% addpath(fullfile(pwd,'functions'))
% addpath(fullfile(pwd,['functions' filesep 'MEX']))

tic
display('---------------------------------')
display('Running the Crack Counter 3D - simulation of microstructure')

%% Input 

% cAgg = 0.2; % volume fraction of aggregates in the mix
% GC_frac = [1,1]*1e-3; % grading cuve - fractions (diameter)
% GC_amount = [1]; % volume of grains from each interval
% 
% cubeSizeRVE = 20e-3;       % size of cubical RVE
% minDistBetweenGrains = 0;  % minimum distance between grains
% periodicBC = true;        % true = grains can intersect the RVE boundary, not available in C_funcions
% 
% C_functions = false; % true = utilizes MEX function written in C++ for placement of grains, plotting of cracks not available!!! Only grains can be plotted.
%                      %        only periodic boundary conditions available
%                      
% figureOut = false; % true = plots the generated microstructure (significantly slows the process!)
%                    % false = no plotting
% saveFigure = false; % true = saves the figure
nDivisions = 20;  % division of the particle boundary for plotting

calculationOfCracks = true; % true = calculates the cracks
% criticalRatio = 0.9;  % critical ratio of gap between aggregates to their mean diameter,
%                       % cannot be > 1, otherwise the cracks across the entire sector
%                       % could be omitted

%% Calculations 

particles = f_grainsGeneration3D(cAgg, cubeSizeRVE^3, GC_frac, GC_amount);
fprintf('%d particles were generated, placement of grains...\n', length(particles))

if C_functions
    placedParticles = f_cpp_placeParticles(particles,cubeSizeRVE,minDistBetweenGrains);
else
    placedParticles = f_placeParticles(particles,cubeSizeRVE,minDistBetweenGrains,periodicBC);
end

if ~calculationOfCracks
    display('100% particles placed.')
else
    display('100% particles placed, calculation of crack density...')
end

if C_functions
    counterCrackLengthToDensity = f_cpp_countCracks(placedParticles,criticalRatio);
else
    counterCrackLengthToDensity = 0;
    displayLimit = 1; % to inform about the progress
    displayStep = 1;  % to inform about the progress
    firstMessage = false; % to inform about the progress

    for j = 1:length(placedParticles)
        center = [placedParticles(j,1),placedParticles(j,2),placedParticles(j,3)];
        radius = placedParticles(j,4);

        % generate the spheres for plotting
        if figureOut
            if j == 1
                x = zeros(nDivisions+1);
                y = zeros(nDivisions+1);
                z = zeros(nDivisions+1);
            end
            counterPsi = 0;
            for psi = 0:pi/nDivisions:pi
                counterPsi = counterPsi+1;

                omega = 0:2*pi/nDivisions:2*pi;
                x(counterPsi,:) = center(1)+radius*cos(omega)*sin(psi);
                y(counterPsi,:) = center(2)+radius*sin(omega)*sin(psi);
                z(counterPsi,:) = center(3)+radius*cos(psi);
            end

            if figureOut
             p = surface(x,y,z,'EdgeColor','none','FaceColor',[0.9 0.9 0.9],'FaceAlpha',0.9);
             hold on
            end
        end

        %% cracks
        if calculationOfCracks
            for m = 1:(j-1)

                % information about progress
                if ~firstMessage
                    fprintf('Cracks evaluated:  0%%\n');
                    firstMessage = true;
                end
                if j/size(placedParticles,1)*100 > displayLimit
                    fprintf('\b\b\b\b')
                    fprintf('%2.d%%\n', displayLimit);
                    displayLimit = displayLimit+displayStep;
                end

                centerOther = [placedParticles(m,1), placedParticles(m,2), placedParticles(m,3)];
                radiusOther = placedParticles(m,4);

                if figureOut
                    % find intersection of line connecting centers with spheres
                    centerDist = sqrt(sum((centerOther-center).^2));
                    lineDeltas = (centerOther-center)/centerDist;
                    intersectionsActual = f_intersectLineSphere([center, lineDeltas], [center, radius]);
                    intersectionsOther = f_intersectLineSphere([centerOther, lineDeltas], [centerOther, radiusOther]);

                    % choose the closest intersections of the line (crack) and spheres (aggregates)
                    distance_1 = sqrt(sum((intersectionsActual(1,:)-intersectionsOther(1,:)).^2));
                    distance_2 = sqrt(sum((intersectionsActual(1,:)-intersectionsOther(2,:)).^2));
                    distance_3 = sqrt(sum((intersectionsActual(2,:)-intersectionsOther(1,:)).^2));
                    distance_4 = sqrt(sum((intersectionsActual(2,:)-intersectionsOther(2,:)).^2));
                    minDist = min([distance_1; distance_2; distance_3; distance_4]);

                    if minDist == distance_1
                        intersectionActual = intersectionsActual(1,:);
                        intersectionOther = intersectionsOther(1,:);
                    elseif minDist == distance_2
                        intersectionActual = intersectionsActual(1,:);
                        intersectionOther = intersectionsOther(2,:);
                    elseif minDist == distance_3
                        intersectionActual = intersectionsActual(2,:);
                        intersectionOther = intersectionsOther(1,:);
                    else
                        intersectionActual = intersectionsActual(2,:);
                        intersectionOther = intersectionsOther(2,:);
                    end
                else
                    % faster, cannot be used in plotting
                    minDist = sqrt(sum((center(1:3)-centerOther(1:3)).^2))-radius-radiusOther; 
                end

                % check whether the crack forms
                meanDiameter = radius+radiusOther;
                geomRatio = minDist/meanDiameter;

                if geomRatio < criticalRatio
                    counterCrackLengthToDensity = counterCrackLengthToDensity+(minDist/2)^3;
                    if figureOut
                        plot3([intersectionActual(1), intersectionOther(1)], [intersectionActual(2), intersectionOther(2)], [intersectionActual(3), intersectionOther(3)], '-r', 'linewidth', 2)
                        hold on
                    end
                end
            end
        end
    end
end

if figureOut
    p = surface(x,y,z,'EdgeColor','none','FaceColor',[0.9 0.9 0.9],'FaceAlpha',0.9);
    hold on
end

if calculationOfCracks
    crackDensity = counterCrackLengthToDensity/(cubeSizeRVE^3);
    fprintf('Crack density is %.3f. \n', crackDensity)
end

if figureOut
    ax = get(p,'Parent');
    set(ax,'XLim', [0 cubeSizeRVE], 'YLim', [0 cubeSizeRVE], 'YLim', [0 cubeSizeRVE], 'PlotBoxAspectRatio', [1 1 1])
    set(ax, 'Position',[0.1 0.1 0.8 0.8])
    set(ax, 'XTick',[],'YTick',[],'ZTick',[])
    box on;
    view(26, 26);
    axis([0 cubeSizeRVE 0 cubeSizeRVE 0 cubeSizeRVE]);
    axis tight
    axis equal
    lightHandles = light;
    set(lightHandles, 'Position', [cubeSizeRVE*2.6, cubeSizeRVE*0.5, cubeSizeRVE*1.1], 'Style', 'local');
    lighting gouraud
    
    if saveFigure
        display('Saving figure...')
        pngOutput = ['3Dmicrostruct_vol_',num2str(cubeSizeRVE^3),'_c_',num2str(cAgg),'.png'];
        epsOutput = ['3Dmicrostruct_vol_',num2str(cubeSizeRVE^3),'_c_',num2str(cAgg),'.eps'];
        eval(['print -dpng output/',pngOutput])
        eval(['print -depsc2 output/',epsOutput])
    end
end

toc
display('---------------------------------')